package org.example.bindings.gui;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;

import net.java.dev.properties.BaseProperty;
import net.java.dev.properties.IndexedProperty;
import net.java.dev.properties.binding.swing.adapters.ComboAndListModel;
import net.java.dev.properties.binding.swing.adapters.SwingBind;
import net.java.dev.properties.container.BeanContainer;
import net.java.dev.properties.container.ObservableIndexed;
import net.java.dev.properties.container.ObservableProperty;
import net.java.dev.properties.events.PropertyListener;

import org.example.bindings.domain.BPPerson;

public class BPBindingsPanel extends BindingsPanel<BPPerson> {

    public final ObservableIndexed<BPPerson> persons = ObservableIndexed.create();
    public final ObservableProperty<BPPerson> selectedPerson = ObservableProperty.create();
    public final ObservableProperty<Boolean> selected = ObservableProperty.create();
    
    public BPBindingsPanel() {
        BeanContainer.bind(this);
    }

    @Override
    protected List<BPPerson> createPersons(String... names) {
        List<BPPerson> result = new ArrayList<BPPerson>();
        for(String name : names) {
            result.add(new BPPerson(name));
        }
        return result;
    }

    @Override
    public void setPersons(List<BPPerson> persons) {
        // ObservableIndexed does not support clear or addAll :(
        //this.persons.clear();
        //this.persons.addAll(persons);
        
        for(int i = 0; i < this.persons.size(); i++) {
            this.persons.remove(i);
        }
        for(int i = 0; i < persons.size(); i++) {
            this.persons.add(persons.get(i));
        }
    }
    
    @Override
    protected void initBindings() {
        // normall you would do this, except we want to update the content sometimes:
//        SwingBind.get().bindContent(persons, list);
        final CustomPropertyListModel<BPPerson> model =
            new CustomPropertyListModel<BPPerson>(persons);
        list.putClientProperty("Property", persons);
        list.setModel(model);
        
        list.setCellRenderer(new DefaultListCellRenderer() {
            @Override
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                String name = value != null ? value.toString() : "";
                if(value instanceof BPPerson) {
                    name = ((BPPerson) value).name.get();
                }
                return super.getListCellRendererComponent(list, 
                        name, index, isSelected, cellHasFocus);
            }
        });
        
        SwingBind.get().bindSelectionItem(selectedPerson, list, true);
        
        // Not sure how this could work better (no ELProperty equivalent?)
        // Based on SwingFactory#bindMasterDetail
//        list.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
//            public void valueChanged(ListSelectionEvent e) {
//                if(!e.getValueIsAdjusting()) {
//                    if(selectedPerson != null) {
//                        SwingBind.get().bind(selectedPerson.get().name, nameTextField);
//                    }
//                    else {
//                        SwingBind.get().unbind(nameTextField);
//                    }
//                }
//            }
//        });

        // the enabled binding works, but only one binding per component
        // allowed. So if we try to bind the name to the text later
        // this one gets removed (and the textfield never gets enabled again).
//        SwingBind.get().bindEnabled(selected, nameTextField);
        
        BeanContainer.get().addListener(selectedPerson, new PropertyListener() {
            PropertyListener listener = new PropertyListener() {
                public void propertyChanged(BaseProperty prop, Object oldValue, Object newValue, int index) {
                    int row = persons.get().indexOf(selectedPerson);
                    model.updateRow(row);
                }
            };
            
            public void propertyChanged(BaseProperty prop, Object oldValue, Object newValue, int index) {
                if(oldValue instanceof BPPerson) {
                    BeanContainer.get().removeListener(((BPPerson) oldValue).name, listener);
                    SwingBind.get().unbind(nameTextField);
                }
                
                if(newValue != null) {
                    SwingBind.get().bind(selectedPerson.get().name, nameTextField);
                    BeanContainer.get().addListener(selectedPerson.get().name,
                            listener);

                }
                selected.set(newValue != null);
            }
        });
    }

     private static class CustomPropertyListModel<T> extends ComboAndListModel<T> {

        public CustomPropertyListModel(IndexedProperty<T> prop, boolean forceNull) {
            super(prop, forceNull);
        }

        public CustomPropertyListModel(IndexedProperty<T> prop) {
            super(prop);
        }

        public void updateRow(int row) {
            fireContentsChanged(this, row, row);
        }
    }
}
