package org.example.bindings.gui;

import java.awt.BorderLayout;
import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingWorker;

import org.example.util.gui.GUIUtils;
import org.example.util.gui.layout.MigPanel;
import org.jdesktop.swingx.JXTitledPanel;

import com.jidesoft.swing.JideSplitPane;

public abstract class BindingsPanel<T> extends JPanel {
    
    protected JList list;
    protected JTextField nameTextField;
    
    public BindingsPanel() {
        super(new BorderLayout());
        list = new JList();
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        nameTextField = new JTextField();
    }
    
    protected void createContentPanel() {
        JXTitledPanel listPanel = new JXTitledPanel("Persons", new JScrollPane(list));

        MigPanel detailsPanel = new MigPanel();
        detailsPanel.addInput("Name", nameTextField);
        
        JideSplitPane pane = new JideSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        pane.add(listPanel);
        pane.add(detailsPanel);
        pane.setProportionalLayout(true);
        pane.setProportions(new double[] { 0.25 });
        pane.setBorder(GUIUtils.createSpacingBorder(5));
        pane.setDividerSize(5);
        
        add(pane);
    }
    
    public void loadData() {
        createContentPanel();
        
        initBindings();
        
        SwingWorker<List<T>, Integer> task = new SwingWorker<List<T>, Integer>() {
            @Override
            protected List<T> doInBackground() throws Exception {
                return createPersons("Walter", "Walter1", "Walter2", "Walter3");
            }
            
            @Override
            protected void done() {
                if(!isCancelled()) {
                    try {
                        setPersons(get());
                    }
                    catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    catch (ExecutionException e) {
                        e.printStackTrace();
                    }
                }
            }
        };
        
        task.execute();
    }

    /**
     * Called on a SwingWorker thread.
     */
    protected abstract List<T> createPersons(String... names);

    public abstract void setPersons(List<T> persons);
    
    protected abstract void initBindings();
}
