package org.example.bindings.gui;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.KeyboardFocusManager;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.logging.Level;

import javax.swing.AbstractAction;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.example.util.gui.GUIUtils;
import org.jdesktop.swingx.JXErrorPane;
import org.jdesktop.swingx.JXFrame;
import org.jdesktop.swingx.error.ErrorInfo;

import com.jidesoft.swing.JideSplitPane;

public class BindingsTest extends JXFrame {

    public static void main(String[] args) {
        Toolkit.getDefaultToolkit().getSystemEventQueue().push(new EventQueue() {
            @Override
            protected void dispatchEvent(AWTEvent event) {
                try {
                    super.dispatchEvent(event);
                }
                catch(final Exception e) {
                    e.printStackTrace();
                    EventQueue.invokeLater(new Runnable() {
                        public void run() {
                            JXErrorPane.showDialog(
                                    KeyboardFocusManager.getCurrentKeyboardFocusManager().getActiveWindow(),
                                    new ErrorInfo(
                                    "Error", "An uncaught exception occured.",
                                    e.getLocalizedMessage(), e.getClass().getSimpleName(),
                                    e, Level.SEVERE, null));
                        }
                    });
                }
              }
        });
        
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                try {
//                    UIManager.setLookAndFeel(NimbusLookAndFeel.class);
                    UIManager.setLookAndFeel(
                            UIManager.getSystemLookAndFeelClassName());
                    
                }
                // yes, multicatch would be nice in Java 7!
                //              catch (ClassNotFoundException e) {
                //              catch (InstantiationException e) {
                //              catch (IllegalAccessException e) {
                //              catch (UnsupportedLookAndFeelException e) {
                catch (Exception e) {
                    e.printStackTrace();
                }

                BindingsTest test = new BindingsTest();
                test.pack();
                test.setExtendedState(MAXIMIZED_BOTH);
                test.setVisible(true);
            }
        });

    }

    public BindingsTest() {
        super("Property bindings test", true);

        setStartPosition(StartPosition.CenterInScreen);
        setPreferredSize(new Dimension(550, 400));
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        JMenu menu = new JMenu(new AbstractAction("File") {
            public void actionPerformed(ActionEvent e) {
            }
        });
        
//        AbstractAction configAction = new AbstractAction("Show config dialog") {
//            public void actionPerformed(ActionEvent e) {
//            }
//        };
//        configAction.setEnabled(false);
//        configAction.putValue(Action.SHORT_DESCRIPTION, "Not yet implemented");
//        menu.add(configAction);
//        menu.addSeparator();
        menu.add(new AbstractAction("Quit") {
            public void actionPerformed(ActionEvent e) {
                dispose();
                System.exit(0);
            }
        });

        JMenuBar bar = new JMenuBar();
        bar.add(menu);
        setJMenuBar(bar);

        final JSR295BindingsPanel jsr295Panel = new JSR295BindingsPanel();
        final BPBindingsPanel bpPanel = new BPBindingsPanel();
        final UIFactoryBPBindingsPanel factoryPanel = new UIFactoryBPBindingsPanel();
        
        final JTabbedPane tabs = new JTabbedPane();
        tabs.addTab("Credits", GUIUtils.createTextArea("/credits.txt", true));
        tabs.addTab("Bean Bindings (JSR295)", jsr295Panel);
        tabs.addTab("Bean properties", bpPanel);
        tabs.addTab("Bean properties (UI factory)", factoryPanel);
        
        final JTabbedPane sourceTabs = new JTabbedPane();
        sourceTabs.addTab("BindingsTestPanel source", GUIUtils.createTextArea(BindingsPanel.class));
        sourceTabs.addTab("JSR295 source", GUIUtils.createTextArea(jsr295Panel.getClass()));
        sourceTabs.addTab("Bean properties source", GUIUtils.createTextArea(bpPanel.getClass()));
        sourceTabs.addTab("BP UI factory source", GUIUtils.createTextArea(factoryPanel.getClass()));
        
        tabs.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                sourceTabs.setSelectedIndex(tabs.getSelectedIndex());
            }
        });
        
        JideSplitPane pane = new JideSplitPane(JideSplitPane.HORIZONTAL_SPLIT);
        pane.add(tabs);
        pane.add(sourceTabs);
        pane.setProportionalLayout(true);
        pane.setProportions(new double[] { 0.50 });
        pane.setDividerSize(5);
        
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(pane, BorderLayout.CENTER);
        
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                // lazy so sub classes can init their fields first
                jsr295Panel.loadData();
                bpPanel.loadData();
                factoryPanel.loadData();
            }
        });
    }
}
