package org.example.bindings.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.java.dev.properties.IndexedProperty;
import net.java.dev.properties.binding.swing.SwingFactory;
import net.java.dev.properties.container.BeanContainer;
import net.java.dev.properties.container.ObservableIndexed;
import net.java.dev.properties.container.ObservableProperty;
import net.java.dev.properties.container.PropertyContext;

import org.example.bindings.domain.BPPerson;
import org.example.util.gui.GUIUtils;
import org.example.util.gui.layout.MigPanel;
import org.jdesktop.swingx.JXTitledPanel;

import com.jidesoft.swing.JideSplitPane;

public class ListUIFactoryBPBindingsPanel extends BindingsPanel<BPPerson> {

    public final ObservableIndexed<BPPerson> persons = ObservableIndexed.create();
    public final ObservableProperty<BPPerson> selectedPerson = ObservableProperty.create();
    
    public ListUIFactoryBPBindingsPanel() {
        BeanContainer.bind(this);
    }

    @Override
    protected List<BPPerson> createPersons(String... names) {
        List<BPPerson> result = new ArrayList<BPPerson>();
        for(String name : names) {
            result.add(new BPPerson(name));
        }
        return result;
    }

    @Override
    public void setPersons(List<BPPerson> persons) {
        // ObservableIndexed does not support clear or addAll :(
        //this.persons.clear();
        //this.persons.addAll(persons);
        
        for(int i = 0; i < this.persons.size(); i++) {
            this.persons.remove(i);
        }
        for(int i = 0; i < persons.size(); i++) {
            this.persons.add(persons.get(i));
        }
    }
    
    @Override
    protected void createContentPanel() {
        SwingFactory factory = new SwingFactory() {
            @Override
            protected void addImpl(JComponent layout, JComponent component, PropertyContext property) {
                JComponent label = createLabel(component, property);
                if(label == null) {
                    label = new JLabel("");
                }
                
                layout.add(label);
                layout.add(component, "pushx, wrap");
            }

            @Override
            public JComponent createLayout(int elements) {
                return new MigPanel();
            }
            
            @Override
            protected JComponent combineMasterDetail(JComponent master, JComponent detail) {
                JideSplitPane pane = new JideSplitPane(JSplitPane.HORIZONTAL_SPLIT);
                pane.add(new JXTitledPanel("Persons", master));
                pane.add(detail);
                pane.setProportionalLayout(true);
                pane.setProportions(new double[] { 0.25 });
                pane.setBorder(GUIUtils.createSpacingBorder(5));
                
                return pane;
            }
        };
        final SwingFactory masterFactory = new ListComponentFactory();

        masterFactory.componentFactory.get().registerIndexedTypeSingle(
                BPPerson.class, JList.class);
        
        JComponent content = factory.createMasterDetail(persons, masterFactory);
        add(content);
    }
    
    @Override
    protected void initBindings() {
        // happens in createContentPanel
    }
    
    private static class ListComponentFactory extends SwingFactory {
        @Override
        protected void addImpl(JComponent layout, JComponent component, PropertyContext property) {
            layout.add(component);
        }

        @Override
        public JComponent createLayout(int elements) {
            return new JPanel(new BorderLayout());
        }
        
        @Override
        protected void bindMasterDetail(JComponent master, final JComponent detail, final IndexedProperty property) {
            componentFactory.get().bindComponentTree(this, property.getParent(), master);
            final JList t = (JList)findComponent(JList.class, master);

            getValidation().trackIndexedValidity(property, t);
            // For some reason this crap clears the selection in the table fucking up
            // everything... No idea???
            /*final PropertyListener tableUpdater = new PropertyListener() {
                public void propertyChanged(BaseProperty prop, Object oldValue, Object newValue, int index) {
                    TableAdapterModel model = (TableAdapterModel)t.getModel();
                    if(oldValue != newValue) {
                        model.fireSafe(index);
                    }
                }
            };*/
            t.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                private Object previousRow = null;
                public void valueChanged(ListSelectionEvent e) {
                    // currently only support single selection
                    if(previousRow != null) {
                        //BeanContainer.get().removeListener(previousRow, tableUpdater);
                        getValidation().stopTracking(previousRow, detail);
                        componentFactory.get().unbindComponentTree(detail);
                        previousRow = null;
                    }
                    
                    int row = t.getSelectedIndex();
                    if(row > -1) {
                        if(!detail.isEnabled()) {
                            enableComponentTree(detail, true);
                        }
                        previousRow = property.get(row);
                        componentFactory.get().bindComponentTree(ListComponentFactory.this, previousRow, detail);
                        //BeanContainer.get().addListener(previousRow, tableUpdater);
                        getValidation().trackValidity(previousRow, detail);
                    } else {
                        enableComponentTree(detail, false);
                    }
                }
            });
        }
        /**
         * Disables or enables the components within a component tree recursively
         */
        private void enableComponentTree(JComponent cmp, boolean enable) {
            cmp.setEnabled(enable);
            for(Component c : cmp.getComponents()) {
                c.setEnabled(enable);
                if(c instanceof JComponent) {
                    enableComponentTree((JComponent)c, enable);
                }
            }
        }
        
        /**
         * Traverses the component tree and finds the given component
         */
        private JComponent findComponent(Class c, JComponent component) {
            for(Component cmp : component.getComponents()) {
                if(c.isAssignableFrom(cmp.getClass())) {
                    return (JComponent)cmp;
                }
                if(cmp instanceof JComponent) {
                    JComponent result = findComponent(c, (JComponent)cmp);
                    if(result != null) {
                        return result;
                    }
                }
            }
            return null;
        }
    };
}
