package org.example.bindings.gui;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JSplitPane;

import net.java.dev.properties.binding.swing.SwingFactory;
import net.java.dev.properties.container.BeanContainer;
import net.java.dev.properties.container.ObservableIndexed;
import net.java.dev.properties.container.PropertyContext;

import org.example.bindings.domain.BPPerson;
import org.example.util.gui.GUIUtils;
import org.example.util.gui.layout.MigPanel;
import org.jdesktop.swingx.JXTitledPanel;

import com.jidesoft.swing.JideSplitPane;

public class UIFactoryBPBindingsPanel extends BindingsPanel<BPPerson> {

    public final ObservableIndexed<BPPerson> persons = ObservableIndexed.create();
    
    public UIFactoryBPBindingsPanel() {
        BeanContainer.bind(this);
    }

    @Override
    protected List<BPPerson> createPersons(String... names) {
        List<BPPerson> result = new ArrayList<BPPerson>();
        for(String name : names) {
            result.add(new BPPerson(name));
        }
        return result;
    }

    @Override
    public void setPersons(List<BPPerson> persons) {
        // ObservableIndexed does not support clear or addAll :(
        //this.persons.clear();
        //this.persons.addAll(persons);
        
        for(int i = 0; i < this.persons.size(); i++) {
            this.persons.remove(i);
        }
        for(int i = 0; i < persons.size(); i++) {
            this.persons.add(persons.get(i));
        }
    }
    
    @Override
    protected void createContentPanel() {
        SwingFactory factory = new SwingFactory() {
            @Override
            protected void addImpl(JComponent layout, JComponent component, PropertyContext property) {
                JComponent label = createLabel(component, property);
                if(label == null) {
                    label = new JLabel("");
                }
                
                layout.add(label);
                layout.add(component, "pushx, wrap");
            }

            @Override
            public JComponent createLayout(int elements) {
                return new MigPanel();
            }
            
            @Override
            protected JComponent combineMasterDetail(JComponent master, JComponent detail) {
                JideSplitPane pane = new JideSplitPane(JSplitPane.HORIZONTAL_SPLIT);
                pane.add(new JXTitledPanel("Persons", master));
                pane.add(detail);
                pane.setProportionalLayout(true);
                pane.setProportions(new double[] { 0.25 });
                pane.setBorder(GUIUtils.createSpacingBorder(5));
                pane.setDividerSize(5);
                
                return pane;
            }
        };
       
        // creates a table, does not support JList as master component
        JComponent content = factory.createMasterDetail(persons, 
                SwingFactory.centerLayoutFactory());
        add(content);
    }
    
    @Override
    protected void initBindings() {
        // happens in createContentPanel
    }
}
