/*
 * Caricature.java
 *
 * Created on April 8, 2006, 4:09 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.example.caricature;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import net.java.dev.properties.BaseProperty;
import net.java.dev.properties.Property;
import net.java.dev.properties.annotations.Attributes;
import net.java.dev.properties.container.BeanContainer;
import net.java.dev.properties.container.ObservableProperty;
import net.java.dev.properties.container.ObservableWrapper;
import net.java.dev.properties.events.PropertyListener;

/**
 *
 * @author sky
 */
public class BPCaricature extends JPanel {
    private static Map<String,Image> imageMap;
    
    public final Property<Boolean> empty = ObservableProperty.create(false);
    public final Property<Integer> mouthStyle = ObservableProperty.create(1);
    public final Property<Integer> faceStyle = ObservableProperty.create(1);
    public final Property<Integer> hairStyle = ObservableProperty.create(1);
    public final Property<Integer> eyeStyle = ObservableProperty.create(1);
    public final Property<Integer> noseStyle = ObservableProperty.create(1);
    public final Property<Integer> rotation = ObservableProperty.create(0);
    public final Property<Float> scale = ObservableProperty.create(0.75f);
    
    @Attributes(displayName="Scale (%)")
    public final Property<Integer> scaleInt = new ObservableWrapper.ReadWrite<Integer>(scale) {
        @Override
        public Integer get() {
            return (int) (scale.get() * 100);
        }
        
        @Override
        public void set(Integer i) {
            scale.set(i.floatValue() / 100);
        }
    };
    
    public BPCaricature() {
        BeanContainer.bind(this);
        
        final PropertyListener repaintListener = new PropertyListener() {
            public void propertyChanged(BaseProperty prop, Object oldValue, Object newValue, int index) {
                BPCaricature.this.repaint();
            }
        };
        
        BeanContainer.get().addListener(this, repaintListener);
        
        if (imageMap == null) {
            imageMap = new HashMap<String,Image>(1);
            for (int i = 1; i < 6; i++) {
                getImage("face", i);
                getImage("hair", i);
                getImage("eyes", i);
                getImage("nose", i);
                getImage("mouth", i);
            }
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        if (!isPreferredSizeSet()) {
            Image image = getImage("mouth", 1);
            return new Dimension(image.getWidth(null), image.getHeight(null));
        }
        return super.getPreferredSize();
    }
    
    @Override
    public Dimension getMaximumSize() {
        if (!isMaximumSizeSet()) {
            return getPreferredSize();
        }
        return super.getMaximumSize();
    }
    
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (empty.get()) {
            return;
        }
        Graphics2D g2 = (Graphics2D)g.create();
//        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
//                RenderingHints.VALUE_ANTIALIAS_ON);
        Image image = getImage("face", faceStyle.get());
        int iw = image.getWidth(null);
//        int ih = image.getHeight(null);
//        g2.translate(iw / 2, ih / 2);
        g2.translate(getWidth() / 2, getHeight() / 2);
        if (iw != getWidth()) {
            float forcedScale = (float)getWidth() / (float)iw;
            g2.scale(forcedScale, forcedScale);
        }
        float scale = this.scale.get();
        if (scale != 1) {
            g2.scale(scale, scale);
        }
        int rotation = this.rotation.get();
        if (rotation != 0) {
            g2.rotate(Math.toRadians(rotation));
        }
        drawImage(g2, "face", faceStyle.get());
        drawImage(g2, "hair", hairStyle.get());
        drawImage(g2, "eyes", eyeStyle.get());
        drawImage(g2, "nose", noseStyle.get());
        drawImage(g2, "mouth", mouthStyle.get());
        g2.dispose();
    }

    private void drawImage(Graphics g, String string, int i) {
        Image image = getImage(string, i);
        g.drawImage(image, -image.getWidth(null) / 2, -image.getHeight(null) / 2, null);
    }

    private Image getImage(String key, int style) {
        String imageName = key + style + ".gif";
        Image image = imageMap.get(imageName);
        if (image == null) {
            System.err.println("name=" + imageName);
            URL imageLoc = getClass().getResource("resources/" + imageName);
            image = new ImageIcon(imageLoc).getImage();
            imageMap.put(imageName, image);
        }
        return image;
    }
}
