package org.example.util.gui;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.Border;

public final class GUIUtils {
    public static Border createSpacingBorder(int size) {
        return BorderFactory.createEmptyBorder(size, size, size, size);
    }

    public static JComponent createTextArea(Class<?> code) {
        return createTextArea(
                "/" + code.getCanonicalName().replace('.', '/') + ".java", false);
    }
    
    public static JComponent createTextArea(String resourcePath, boolean wrap) {
        JTextArea textArea = new JTextArea();
        textArea.setWrapStyleWord(wrap);
        textArea.setLineWrap(wrap);
        textArea.setEditable(false);
        
        InputStream creditsStream = GUIUtils.class.getResourceAsStream(resourcePath);
        if(creditsStream != null) {
            BufferedReader reader = new BufferedReader(new InputStreamReader(creditsStream));
            String line = null;
            try {
                while ((line = reader.readLine()) != null) {
                    textArea.append(line);
                    textArea.append("\n");
                }
            }
            catch (IOException e1) {
                e1.printStackTrace();
                textArea.append(GUIUtils.stacktraceToString(e1));
            }
            finally {
                try {
                    reader.close();
                }
                catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
        else {
            textArea.append("Could not find resource: " + resourcePath);
        }

        textArea.setCaretPosition(0);
        
        return new JScrollPane(textArea);
    }

    public static String stacktraceToString(Throwable e) {
        if (e != null) {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            pw.flush();
            String mess = sw.toString();
            return mess;
        }
        else {
            return "";
        }
    }
}
